/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 10 janvier 2007
   derniere modification le 12/07/2007 */

#include <GL/glew.h>

#include <SCE/core/SCECMatrix.h>


/* ajoute le 02/07/2007 */
void SCE_CLoadIdentityMatrix(const GLenum matrix)
{
    GLint mode;

    if(matrix == SCE_CURRENT_MATRIX)
        glLoadIdentity();
    else
    {
        glGetIntegerv(GL_MATRIX_MODE, &mode);
        glMatrixMode(matrix);
        glLoadIdentity();
        glMatrixMode(mode);
    }
}

/* ajoute le 02/07/2007 */
/* revise le 12/07/2007 */
void SCE_CLoadIdentitySetMatrix(const GLenum matrix)
{
    if(matrix != SCE_CURRENT_MATRIX)
        glMatrixMode(matrix);
    glLoadIdentity();
}

void SCE_CPushMatrix(const GLenum matrix)
{
    GLint mode;

    if(matrix == SCE_CURRENT_MATRIX)
        glPushMatrix();
    else
    {
        glGetIntegerv(GL_MATRIX_MODE, &mode);
        glMatrixMode(matrix);
        glPushMatrix();
        glMatrixMode(mode);
    }
}

void SCE_CPopMatrix(const GLenum matrix)
{
    GLint mode;

    if(matrix == SCE_CURRENT_MATRIX)
        glPopMatrix();
    else
    {
        glGetIntegerv(GL_MATRIX_MODE, &mode);
        glMatrixMode(matrix);
        glPopMatrix();
        glMatrixMode(mode);
    }
}

/* revise le 12/07/2007 */
void SCE_CPushSetMatrix(const GLenum matrix)
{
    if(matrix != SCE_CURRENT_MATRIX)
        glMatrixMode(matrix);
    glPushMatrix();
}

/* revise le 12/07/2007 */
void SCE_CPopSetMatrix(const GLenum matrix)
{
    if(matrix != SCE_CURRENT_MATRIX)
        glMatrixMode(matrix);
    glPopMatrix();
}

void SCE_CSetMatrixMode(const GLenum matrix)
{
    glMatrixMode (matrix);
}

GLint SCE_CGetMatrixMode(void)
{
    GLint matrix;
    glGetIntegerv(GL_MATRIX_MODE, &matrix);
    return matrix;
}

void SCE_CMultMatrix(const GLenum matrixmode, const float *m)
{
    GLint backmode;

    if(matrixmode == SCE_CURRENT_MATRIX)
        glMultMatrixf(m);
    else
    {
        glGetIntegerv(GL_MATRIX_MODE, &backmode);
        glMatrixMode(matrixmode);
        glMultMatrixf(m);
        glMatrixMode(backmode);
    }
}

/* revise le 12/07/2007 */
void SCE_CLoadMatrix(const GLenum matrixmode, const float *m)
{
    GLint backmode;

    if(matrixmode == SCE_CURRENT_MATRIX)
        glLoadMatrixf(m);
    else
    {
        glGetIntegerv(GL_MATRIX_MODE, &backmode);
        glMatrixMode(matrixmode);
        glLoadMatrixf(m);
        glMatrixMode(backmode);
    }
}


/* revise le 12/07/2007 */
void SCE_CMultSetMatrix(const GLenum matrixmode, const float *m)
{
    if(matrixmode != SCE_CURRENT_MATRIX)
        glMatrixMode(matrixmode);
    glMultMatrixf(m);
}

/* revise le 12/07/2007 */
void SCE_CLoadSetMatrix(const GLenum matrixmode, const float *m)
{
    if (matrixmode != SCE_CURRENT_MATRIX)
        glMatrixMode(matrixmode);
    glLoadMatrixf(m);
}


void SCE_CGetMatrix(int matrix, float *m)
{
    if (matrix == SCE_CURRENT_MATRIX)
        glGetIntegerv (GL_MATRIX_MODE, &matrix);

    switch (matrix)
    {
    case GL_MODELVIEW:
        glGetFloatv (GL_MODELVIEW_MATRIX, m);
        break;
    case GL_PROJECTION:
        glGetFloatv (GL_PROJECTION_MATRIX, m);
        break;
    case GL_TEXTURE:
        glGetFloatv (GL_TEXTURE_MATRIX, m);
        break;
    }
}


void SCE_CTranslateMatrix(float x, float y, float z)
{
    glTranslatef(x, y, z);
}

void SCE_CRotateMatrix(float angle, float x, float y, float z)
{
    glRotatef(angle, x, y, z);
}

void SCE_CScaleMatrix(float x, float y, float z)
{
    glScalef(x, y, z);
}


void SCE_CTranslateMatrixv(float *v)
{
    glTranslatef(v[0], v[1], v[2]);
}

void SCE_CRotateMatrixv(float angle, float *v)
{
    glRotatef(angle, v[0], v[1], v[2]);
}

void SCE_CScaleMatrixv(float *v)
{
    glScalef(v[0], v[1], v[2]);
}



void SCE_CLook3D(
  float p0, float p1, float p2,
  float v0, float v1, float v2,
  float i0, float i1, float i2)
{
    gluLookAt(p0, p1, p2, v0, v1, v2, i0, i1, i2);
}


void SCE_CLook3Dv(const float *p, const float *v, const float *i)
{
    gluLookAt(p[0], p[1], p[2], v[0], v[1], v[2], i[0], i[1], i[2]);
}


void SCE_CSetFrustumMatrix(
  GLdouble left, GLdouble right,
  GLdouble bottom, GLdouble top,
  GLdouble near, GLdouble far)
{
    glFrustum(left, right, bottom, top, near, far);
}

void SCE_CSetOrthoMatrix(
  GLdouble left, GLdouble right,
  GLdouble bottom, GLdouble top,
  GLdouble near, GLdouble far)
{
    glOrtho(left, right, bottom, top, near, far);
}


void SCE_CViewport(int x, int y, int w, int h)
{
    glViewport(x, y, w, h);
}
